# Components enabled if supported by target architecture:
%ifarch %ix86 x86_64
  %bcond_without gold
%else
  %bcond_with gold
%endif

%global maj_ver 6
%global min_ver 0
%global patch_ver 0 

Name:          llvm%{maj_ver}
Version:       %{maj_ver}.%{min_ver}.%{patch_ver}
Release:       4%{?dist}
Summary:       The Low Level Virtual Machine Version %{maj_ver}

License:	NCSA
URL:		http://llvm.org
Source0:	http://llvm.org/releases/%{version}/llvm-%{version}%{?rc_ver:rc%{rc_ver}}.src.tar.xz

# recognize s390 as SystemZ when configuring build
Patch0:		llvm-3.7.1-cmake-s390.patch
Patch3:		0001-CMake-Split-static-library-exports-into-their-own-ex.patch
Patch7:		0001-Filter-out-cxxflags-not-supported-by-clang.patch
Patch9:		0001-Export-LLVM_DYLIB_COMPONENTS-in-LLVMConfig.cmake.patch

# Julia patches
Patch101:	llvm-D27629-AArch64-large_model_4.0.patch
Patch102:	llvm-D34078-vectorize-fdiv.patch
Patch103:	llvm-6.0-NVPTX-addrspaces.patch
Patch104:	llvm-D42262-jumpthreading-not-i1.patch
Patch105:	llvm-PPC-addrspaces.patch
Patch106:	llvm-D42260.patch
Patch107:	llvm-rL326843-missing-header.patch
Patch108:	llvm-6.0-r327540.patch
Patch109:	llvm-6.0.0_D27296-libssp.patch
Patch110:	llvm-D45008.patch
Patch111:	llvm-D45070.patch
Patch112:	llvm-6.0.0-ifconv-D45819.patch
Patch113:	llvm-D46460.patch
Patch114:	llvm-rL332680.patch
Patch115:	llvm-rL332682.patch
Patch116:	llvm-rL332302.patch
Patch117:	llvm-rL332694.patch
Patch118:	llvm-rL327898.patch
Patch120:	llvm-6.0-DISABLE_ABI_CHECKS.patch
Patch121:	llvm-OProfile-line-num.patch
Patch122:	llvm-D44892-Perf-integration.patch
Patch123:	llvm-D49832-SCEVPred.patch
Patch124:	llvm-rL323946-LSRTy.patch
Patch125:	llvm-D50010-VNCoercion-ni.patch
Patch126:	llvm-D50167-scev-umin.patch

BuildRequires:  gcc
BuildRequires:  gcc-c++
%if 0%{?fedora} >= 23
BuildRequires: cmake
%else
BuildRequires: cmake3
%endif
BuildRequires:	ninja-build
BuildRequires:	zlib-devel
BuildRequires:  libffi-devel
BuildRequires:	ncurses-devel
%if 0%{?fedora} >= 23
BuildRequires:	python3-sphinx
%endif
BuildRequires:	multilib-rpm-config
%if %{with gold}
BuildRequires:  binutils-devel
%endif
BuildRequires:  libstdc++-static
# Enable extra functionality when run the LLVM JIT under valgrind.
BuildRequires:  valgrind-devel
# LLVM's LineEditor library will use libedit if it is available.
BuildRequires:  libedit-devel

Requires:	%{name}-libs%{?_isa} = %{version}-%{release}

%description
LLVM is a compiler infrastructure designed for compile-time, link-time,
runtime, and idle-time optimization of programs from arbitrary programming
languages. The compiler infrastructure includes mirror sets of programming
tools as well as libraries with equivalent functionality.

This package contains LLVM %{maj_ver} and can be installed
in parallel with other LLVM versions.

%package devel
Summary:	Libraries and header files for LLVM %{maj_ver}
Requires:	%{name}%{?_isa} = %{version}-%{release}
# The installed LLVM cmake files will add -ledit to the linker flags for any
# app that requires the libLLVMLineEditor, so we need to make sure
# libedit-devel is available.
Requires:	libedit-devel
Requires(post): %{_sbindir}/alternatives
Requires(postun): %{_sbindir}/alternatives

%description devel
This package contains library and header files needed to develop new native
programs that use the LLVM infrastructure.

This package contains LLVM %{maj_ver} and can be installed
in parallel with other LLVM versions.

%package doc
Summary:	Documentation for LLVM %{maj_ver}
BuildArch:	noarch
Requires:	%{name} = %{version}-%{release}

%description doc
Documentation for the LLVM compiler infrastructure.

This package contains LLVM %{maj_ver} and can be installed
in parallel with other LLVM versions.

%package libs
Summary:	LLVM %{maj_ver} shared libraries

%description libs
Shared libraries for the LLVM compiler infrastructure.

This package contains LLVM %{maj_ver} and can be installed
in parallel with other LLVM versions.

%package static
Summary:	LLVM %{maj_ver} static libraries

%description static
Static libraries for the LLVM compiler infrastructure.

This package contains LLVM %{maj_ver} and can be installed
in parallel with other LLVM versions.

%prep
%autosetup -n llvm-%{version}%{?rc_ver:rc%{rc_ver}}.src -p1

%build
mkdir -p _build
cd _build

%ifarch s390 %{arm} %ix86
# Decrease debuginfo verbosity to reduce memory consumption during final library linking
%global optflags %(echo %{optflags} | sed 's/-g /-g1 /')
%endif

# Temporary workaround for https://github.com/JuliaLang/julia/issues/27118
%global optflags %(echo %{optflags} | sed 's/-Wp,-D_GLIBCXX_ASSERTIONS //')

# libs as cmake macros turns it on.
%if 0%{?fedora} >= 23
%cmake .. \
%else
/usr/bin/cmake3 .. -DCMAKE_C_FLAGS_RELEASE:STRING=-DNDEBUG -DCMAKE_CXX_FLAGS_RELEASE:STRING=-DNDEBUG \
       -DCMAKE_Fortran_FLAGS_RELEASE:STRING=-DNDEBUG -DCMAKE_VERBOSE_MAKEFILE:BOOL=ON \
       -DCMAKE_INSTALL_PREFIX:PATH=/usr -DINCLUDE_INSTALL_DIR:PATH=/usr/include \
       -DLIB_INSTALL_DIR:PATH=/usr/lib64 -DSYSCONF_INSTALL_DIR:PATH=/etc \
       -DSHARE_INSTALL_PREFIX:PATH=/usr/share -DLIB_SUFFIX=64 \
%endif
	-G Ninja \
	-DBUILD_SHARED_LIBS:BOOL=OFF \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
%ifarch s390 %{arm} %ix86
	-DCMAKE_C_FLAGS_RELWITHDEBINFO="%{optflags} -DNDEBUG" \
	-DCMAKE_CXX_FLAGS_RELWITHDEBINFO="%{optflags} -DNDEBUG" \
%endif
	\
	-DLLVM_TARGETS_TO_BUILD="X86;AMDGPU;PowerPC;NVPTX;SystemZ;AArch64;ARM;Mips;BPF" \
	-DLLVM_ENABLE_LIBCXX:BOOL=OFF \
	-DLLVM_ENABLE_ZLIB:BOOL=ON \
	-DLLVM_ENABLE_FFI:BOOL=ON \
	-DLLVM_ENABLE_RTTI:BOOL=ON \
%if %{with gold}
	-DLLVM_BINUTILS_INCDIR=%{_includedir} \
%endif
	\
	-DLLVM_BUILD_RUNTIME:BOOL=ON \
	\
	-DLLVM_INCLUDE_TOOLS:BOOL=ON \
	-DLLVM_BUILD_TOOLS:BOOL=ON \
	\
	-DLLVM_INCLUDE_TESTS:BOOL=ON \
	-DLLVM_BUILD_TESTS:BOOL=ON \
	\
	-DLLVM_INCLUDE_EXAMPLES:BOOL=ON \
	-DLLVM_BUILD_EXAMPLES:BOOL=OFF \
	\
	-DLLVM_INCLUDE_UTILS:BOOL=ON \
	-DLLVM_INSTALL_UTILS:BOOL=ON \
	-DLLVM_UTILS_INSTALL_DIR:PATH=%{buildroot}%{_libdir}/%{name} \
	\
	-DLLVM_INCLUDE_DOCS:BOOL=ON \
%if 0%{?fedora} >= 23
	-DLLVM_BUILD_DOCS:BOOL=ON \
	-DLLVM_ENABLE_SPHINX:BOOL=ON \
%else
	-DLLVM_BUILD_DOCS:BOOL=OFF \
	-DLLVM_ENABLE_SPHINX:BOOL=OFF \
%endif
	-DLLVM_ENABLE_DOXYGEN:BOOL=OFF \
	-DSPHINX_OUTPUT_HTML:BOOL=OFF \
	-DSPHINX_WARNINGS_AS_ERRORS:BOOL=OFF \
	\
	-DLLVM_BUILD_LLVM_DYLIB:BOOL=ON \
	-DLLVM_DYLIB_EXPORT_ALL:BOOL=ON \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DLLVM_BUILD_EXTERNAL_COMPILER_RT:BOOL=ON \
	-DLLVM_INSTALL_TOOLCHAIN_ONLY:BOOL=OFF \
	\
	-DSPHINX_WARNINGS_AS_ERRORS=OFF \
	-DCMAKE_INSTALL_PREFIX=%{buildroot}/%{_libdir}/%{name} \
	-DSPHINX_EXECUTABLE=%{_bindir}/sphinx-build-3

ninja-build -v

%install
cd _build
ninja-build -v install

cd -

# Move and symlink into FHS dirs
mkdir -p %{buildroot}%{_bindir}
for bin in %{buildroot}%{_libdir}/%{name}/bin/*
do
  # Cannot move llvm-config due to runtime prefix determination
  [ ${bin##*/} = llvm-config ] && continue
  # Already versioned binaries
  if [ ${bin%%%{maj_ver}} != $bin ]
  then
    mv $bin %{buildroot}%{_bindir}/${bin##*/}
    ln -s ../../../bin/${bin##*/} %{buildroot}%{_libdir}/%{name}/bin/${bin##*/}
  else
    # Unversioned binaries
    if [ -L $bin ]
    then
      target=$(readlink $bin)
      # Make the link point to the versioned binary if needed
      [ ${target%%%{maj_ver}} == $target ] && ln -sf ${target}-%{maj_ver} $bin
      [ $target == clang-%{maj_ver} ] && continue
    fi
    mv $bin %{buildroot}%{_bindir}/${bin##*/}-%{maj_ver}
    ln -s ../../../bin/${bin##*/}-%{maj_ver} %{buildroot}%{_libdir}/%{name}/bin/${bin##*/}
  fi
done
for dir in include
do
  mkdir -p %{buildroot}%{_prefix}/$dir/%{name}
  mv %{buildroot}%{_libdir}/%{name}/$dir/* %{buildroot}%{_prefix}/$dir/%{name}/
  rmdir %{buildroot}%{_libdir}/%{name}/$dir
  ln -s ../../$dir/%{name} %{buildroot}%{_libdir}/%{name}/$dir
done
mkdir -p %{buildroot}%{_libdir}/cmake
mv %{buildroot}%{_libdir}/%{name}/lib/cmake/llvm %{buildroot}%{_libdir}/cmake/%{name}
ln -s ../../../cmake/%{name} %{buildroot}%{_libdir}/%{name}/lib/cmake/llvm
mkdir -p %{buildroot}%{_datadir}
%if 0%{?fedora} >= 23
mv %{buildroot}%{_libdir}/%{name}/share/man %{buildroot}%{_datadir}/
%endif

# fix multi-lib

%multilib_fix_c_header --file %{_includedir}/%{name}/llvm/Config/llvm-config.h

# Create ld.so.conf.d entry
mkdir -p %{buildroot}%{_sysconfdir}/ld.so.conf.d
cat >> %{buildroot}%{_sysconfdir}/ld.so.conf.d/%{name}-%{_arch}.conf << EOF
%{_libdir}/%{name}/lib
EOF

%if 0%{?fedora} >= 23
# suffix mandir files with major version to avoid conflict with llvm
for i in %{buildroot}%{_mandir}/man1/*; do
       mv $i "${i%.*}-%{maj_ver}.1"
done
%endif

%check
cd _build
ninja check-all || :

%post libs -p /sbin/ldconfig
%postun libs -p /sbin/ldconfig

%files
%{_bindir}/*
%dir %{_libdir}/%{name}/bin
%{_libdir}/%{name}/bin/*
%exclude %{_libdir}/%{name}/bin/llvm-config
%{_libdir}/%{name}
%exclude %{_libdir}/%{name}/lib
%exclude %{_libdir}/%{name}/include
%if 0%{?fedora} >= 23
%{_mandir}/man1/*.1.*
%exclude %{_mandir}/man1/llvm-config-%{maj_ver}.1.*
%endif
%{_libdir}/%{name}/share/opt-viewer
%license LICENSE.TXT

%files libs
%config(noreplace) %{_sysconfdir}/ld.so.conf.d/%{name}-%{_arch}.conf
%dir %{_libdir}/%{name}
%dir %{_libdir}/%{name}/lib
%{_libdir}/%{name}/lib/BugpointPasses.so
%{_libdir}/%{name}/lib/LLVMHello.so
%if %{with gold}
%{_libdir}/%{name}/lib/LLVMgold.so
%endif
%{_libdir}/%{name}/lib/libLLVM-%{maj_ver}*.so
%{_libdir}/%{name}/lib/libLTO.so.%{maj_ver}*
%license LICENSE.TXT

%files devel
%{_libdir}/%{name}/bin/llvm-config
%if 0%{?fedora} >= 23
%{_mandir}/man1/llvm-config-%{maj_ver}.1.*
%endif
%{_includedir}/%{name}/llvm
%{_includedir}/%{name}/llvm-c
%{_libdir}/%{name}/include
%{_libdir}/%{name}/lib/libLLVM.so
%{_libdir}/%{name}/lib/libLTO.so
%{_libdir}/cmake/%{name}
%{_libdir}/%{name}/lib/cmake

%files static
%{_libdir}/%{name}/lib/*.a
%{_libdir}/%{name}/lib/cmake/llvm/LLVMStaticExports.cmake

%changelog
* Sat Aug 18 2018 Milan Bouchet-Valat <nalimilan@club.fr> - 6.0.1-4
- Add more patches needed to build Julia.
- Remove -D_GLIBCXX_ASSERTIONS from flags, as it makes the Julia build fail.

* Thu Jun 28 2018 Milan Bouchet-Valat <nalimilan@club.fr> - 6.0.1-3
- Add more patches needed by Julia.

* Sun May 13 2018 Milan Bouchet-Valat <nalimilan@club.fr> - 6.0.1-1
- Create versioned llvm6 package based on llvm3.9.

* Thu May 10 2018 Tom Stellard <tstellar@redhat.com> - 6.0.1-0.1.rc1
- 6.0.1 rc1

* Tue Mar 27 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-11
- Re-enable arm tests that used to hang

* Thu Mar 22 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-10
- Fix testcase in backported patch

* Tue Mar 20 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-9
- Prevent external projects from linking against both static and shared
  libraries.  rhbz#1558657

* Mon Mar 19 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-8
- Backport r327651 from trunk rhbz#1554349

* Fri Mar 16 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-7
- Filter out cxxflags and cflags from llvm-config that aren't supported by clang
- rhbz#1556980

* Wed Mar 14 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-6
- Enable symbol versioning in libLLVM.so

* Wed Mar 14 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-5
- Stop statically linking libstdc++.  This is no longer required by Steam
  client, but the steam installer still needs a work-around which should
  be handled in the steam package.
* Wed Mar 14 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-4
- s/make check/ninja check/

* Fri Mar 09 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-3
- Backport fix for compile time regression on rust rhbz#1552915

* Thu Mar 08 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-2
- Build with Ninja: This reduces RPM build time on a 6-core x86_64 builder
  from 82 min to 52 min.

* Thu Mar 08 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-1
- 6.0.0 Release

* Thu Mar 08 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-0.5.rc2
- Reduce debuginfo size on i686 to avoid OOM errors during linking

* Fri Feb 09 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-0.4.rc2
- 6.0.1 rc2

* Fri Feb 09 2018 Igor Gnatenko <ignatenkobrain@fedoraproject.org> - 6.0.0-0.3.rc1
- Escape macros in %%changelog

* Thu Feb 08 2018 Fedora Release Engineering <releng@fedoraproject.org> - 6.0.0-0.2.rc1
- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild

* Fri Jan 19 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-0.1.rc1
- 6.0.1 rc1

* Tue Dec 19 2017 Tom Stellard <tstellar@redhat.com> - 5.0.1-1
- 5.0.1 Release

* Mon Nov 20 2017 Tom Stellard <tstellar@redhat.com> - 5.0.0-5
- Backport debuginfo fix for rust

* Fri Nov 03 2017 Tom Stellard <tstellar@redhat.com> - 5.0.0-4
- Reduce debuginfo size for ARM

* Tue Oct 10 2017 Tom Stellard <tstellar@redhat.com> - 5.0.0-2
- Reduce memory usage on ARM by disabling debuginfo and some non-ARM targets.

* Mon Sep 25 2017 Tom Stellard <tstellar@redhat.com> - 5.0.0-1
- 5.0.0 Release

* Mon Sep 18 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-6
- Add Requires: libedit-devel for llvm-devel

* Fri Sep 08 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-5
- Enable libedit backend for LineEditor API

* Fri Aug 25 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-4
- Enable extra functionality when run the LLVM JIT under valgrind.

* Thu Aug 03 2017 Fedora Release Engineering <releng@fedoraproject.org> - 4.0.1-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Binutils_Mass_Rebuild

* Wed Jul 26 2017 Fedora Release Engineering <releng@fedoraproject.org> - 4.0.1-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Mass_Rebuild

* Wed Jun 21 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-1
- 4.0.1 Release

* Thu Jun 15 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-6
- Install llvm utils

* Thu Jun 08 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-5
- Fix docs-llvm-man target

* Mon May 01 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-4
- Make cmake files no longer depend on static libs (rhbz 1388200)

* Tue Apr 18 2017 Josh Stone <jistone@redhat.com> - 4.0.0-3
- Fix computeKnownBits for ARMISD::CMOV (rust-lang/llvm#67)

* Mon Apr 03 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-2
- Simplify spec with rpm macros.

* Thu Mar 23 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-1
- LLVM 4.0.0 Final Release

* Wed Mar 22 2017 tstellar@redhat.com - 3.9.1-6
- Fix %%postun sep for -devel package.

* Mon Mar 13 2017 Tom Stellard <tstellar@redhat.com> - 3.9.1-5
- Disable failing tests on ARM.

* Sun Mar 12 2017 Peter Robinson <pbrobinson@fedoraproject.org> 3.9.1-4
- Fix missing mask on relocation for aarch64 (rhbz 1429050)

* Wed Mar 01 2017 Dave Airlie <airlied@redhat.com> - 3.9.1-3
- revert upstream radeonsi breaking change.

* Thu Feb 23 2017 Josh Stone <jistone@redhat.com> - 3.9.1-2
- disable sphinx warnings-as-errors

* Fri Feb 10 2017 Orion Poplawski <orion@cora.nwra.com> - 3.9.1-1
- llvm 3.9.1

* Fri Feb 10 2017 Fedora Release Engineering <releng@fedoraproject.org> - 3.9.0-8
- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_Mass_Rebuild

* Tue Nov 29 2016 Josh Stone <jistone@redhat.com> - 3.9.0-7
- Apply backports from rust-lang/llvm#55, #57

* Tue Nov 01 2016 Dave Airlie <airlied@gmail.com - 3.9.0-6
- rebuild for new arches

* Wed Oct 26 2016 Dave Airlie <airlied@redhat.com> - 3.9.0-5
- apply the patch from -4

* Wed Oct 26 2016 Dave Airlie <airlied@redhat.com> - 3.9.0-4
- add fix for lldb out-of-tree build

* Mon Oct 17 2016 Josh Stone <jistone@redhat.com> - 3.9.0-3
- Apply backports from rust-lang/llvm#47, #48, #53, #54

* Sat Oct 15 2016 Josh Stone <jistone@redhat.com> - 3.9.0-2
- Apply an InstCombine backport via rust-lang/llvm#51

* Wed Sep 07 2016 Dave Airlie <airlied@redhat.com> - 3.9.0-1
- llvm 3.9.0
- upstream moved where cmake files are packaged.
- upstream dropped CppBackend

* Wed Jul 13 2016 Adam Jackson <ajax@redhat.com> - 3.8.1-1
- llvm 3.8.1
- Add mips target
- Fix some shared library mispackaging

* Tue Jun 07 2016 Jan Vcelak <jvcelak@fedoraproject.org> - 3.8.0-2
- fix color support detection on terminal

* Thu Mar 10 2016 Dave Airlie <airlied@redhat.com> 3.8.0-1
- llvm 3.8.0 release

* Wed Mar 09 2016 Dan Horák <dan[at][danny.cz> 3.8.0-0.3
- install back memory consumption workaround for s390

* Thu Mar 03 2016 Dave Airlie <airlied@redhat.com> 3.8.0-0.2
- llvm 3.8.0 rc3 release

* Fri Feb 19 2016 Dave Airlie <airlied@redhat.com> 3.8.0-0.1
- llvm 3.8.0 rc2 release

* Tue Feb 16 2016 Dan Horák <dan[at][danny.cz> 3.7.1-7
- recognize s390 as SystemZ when configuring build

* Sat Feb 13 2016 Dave Airlie <airlied@redhat.com> 3.7.1-6
- export C++ API for mesa.

* Sat Feb 13 2016 Dave Airlie <airlied@redhat.com> 3.7.1-5
- reintroduce llvm-static, clang needs it currently.

* Fri Feb 12 2016 Dave Airlie <airlied@redhat.com> 3.7.1-4
- jump back to single llvm library, the split libs aren't working very well.

* Fri Feb 05 2016 Dave Airlie <airlied@redhat.com> 3.7.1-3
- add missing obsoletes (#1303497)

* Thu Feb 04 2016 Fedora Release Engineering <releng@fedoraproject.org> - 3.7.1-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_24_Mass_Rebuild

* Thu Jan 07 2016 Jan Vcelak <jvcelak@fedoraproject.org> 3.7.1-1
- new upstream release
- enable gold linker

* Wed Nov 04 2015 Jan Vcelak <jvcelak@fedoraproject.org> 3.7.0-100
- fix Requires for subpackages on the main package

* Tue Oct 06 2015 Jan Vcelak <jvcelak@fedoraproject.org> 3.7.0-100
- initial version using cmake build system
